from django.conf import settings

from assetfiles import assets
from assetfiles.tests.base import AssetfilesTestCase, Filter1, Filter2
import assetfiles.settings


class TestAssets(AssetfilesTestCase):
    def setUp(self):
        super(TestAssets, self).setUp()
        self.old_filters = assetfiles.settings.FILTERS
        assetfiles.settings.FILTERS = (
            'assetfiles.tests.base.Filter1',
            'assetfiles.tests.base.Filter2',
        )

    def tearDown(self):
        assetfiles.settings.FILTERS = self.old_filters

    def test_find_asset(self):
        path1 = self.mkfile('static/some/dir/main.out.in')
        path2 = self.mkfile('static/another/dir/main.in2')
        asset_path1, filter1 = assets.find('some/dir/main.out')
        asset_path2, filter2 = assets.find('another/dir/main.out2')
        asset_path3, filter3 = assets.find('non/existent/file.out')
        self.assertEquals(path1, asset_path1)
        self.assertIsInstance(filter1, Filter1)
        self.assertEquals(path2, asset_path2)
        self.assertIsInstance(filter2, Filter2)
        self.assertEquals(None, asset_path3)
        self.assertEquals(None, filter3)



import os
import shutil
import tempfile

from assetfiles import assets, filters, settings
from assetfiles.filters.base import BaseFilter
from assetfiles.filters.coffee import CoffeeScriptFilterError
from assetfiles.filters.sass import SassFilterError
from assetfiles.tests.base import (AssetfilesTestCase,
                                   ReplaceFilter, Filter1, Filter2)


class TestBaseFilter(AssetfilesTestCase):
    def test_filters_a_single_input_file(self):
        filter = ReplaceFilter('Hello', 'World')
        path = self.mkfile('main.css', 'Hello')
        result = filter.filter(path)
        self.assertEquals('World', result)

    def test_matches_input_file_by_ext(self):
        filter = ReplaceFilter()
        path1 = os.path.join(self.root, 'main.foo')
        path2 = os.path.join(self.root, 'main.baz')
        path3 = os.path.join(self.root, 'main.plugin.foo')
        path4 = os.path.join(self.root, 'main.css')
        self.assertTrue(filter.matches_input(path1))
        self.assertTrue(filter.matches_input(path2))
        self.assertTrue(filter.matches_input(path3))
        self.assertFalse(filter.matches_input(path4))

    def test_does_not_match_input_file_without_input_exts(self):
        filter = ReplaceFilter()
        filter.input_exts = ()
        path1 = os.path.join(self.root, 'main.foo')
        path2 = os.path.join(self.root, 'main.baz')
        self.assertFalse(filter.matches_input(path1))
        self.assertFalse(filter.matches_input(path2))

    def test_set_single_input_ext(self):
        filter = ReplaceFilter()
        filter.input_ext = 'foo'
        self.assertEquals(('foo',), filter.input_exts)

    def test_matches_output_file_by_ext(self):
        filter = ReplaceFilter()
        path1 = os.path.join(self.root, 'main.bar')
        path2 = os.path.join(self.root, 'main.plugin.bar')
        path3 = os.path.join(self.root, 'main.foo')
        self.assertTrue(filter.matches_output(path1))
        self.assertTrue(filter.matches_output(path2))
        self.assertFalse(filter.matches_output(path3))

    def test_does_not_match_output_file_without_output_ext(self):
        filter = ReplaceFilter()
        filter.output_ext = None
        path1 = os.path.join(self.root, 'main.bar')
        self.assertFalse(filter.matches_output(path1))

    def test_returns_possible_input_paths(self):
        filter = ReplaceFilter()
        self.assertEquals(set([
            'dir/main.bar.foo',
            'dir/main.foo',
            'dir/main.bar.baz',
            'dir/main.baz',
        ]), filter.possible_input_paths('dir/main.bar'))
        self.assertEquals(set([
            'dir/main.plugin.bar.foo',
            'dir/main.plugin.foo',
            'dir/main.plugin.bar.baz',
            'dir/main.plugin.baz',
        ]), filter.possible_input_paths('dir/main.plugin.bar'))

    def test_returns_output_path(self):
        filter = ReplaceFilter()
        self.assertEquals('dir/main.bar', filter.output_path('dir/main.foo'))
        self.assertEquals('dir/main.bar', filter.output_path('dir/main.bar.foo'))
        self.assertEquals('dir/main.plugin.bar', filter.output_path('dir/main.plugin.foo'))


class TestFilters(AssetfilesTestCase):
    def setUp(self):
        super(TestFilters, self).setUp()
        self.old_filters = settings.FILTERS
        settings.FILTERS = (
            'assetfiles.tests.base.Filter1',
            'assetfiles.tests.base.Filter2',
        )

    def tearDown(self):
        settings.FILTERS = self.old_filters

    def test_filter_from_path(self):
        self.assertIsInstance(filters.find_by_input_path('main.in'), Filter1)
        self.assertIsInstance(filters.find_by_input_path('main.in1'), Filter1)
        self.assertIsInstance(filters.find_by_input_path('main.in2'), Filter2)
        self.assertEquals(None, filters.find_by_input_path('main.out'))

    def test_filter_to_path(self):
        self.assertIsInstance(filters.find_by_output_path('main.out'), Filter1)
        self.assertIsInstance(filters.find_by_output_path('main.out2'), Filter2)
        self.assertEquals(None, filters.find_by_output_path('main.in'))


def filter(path):
    asset_path, filter = assets.find(path)
    return filter.filter(asset_path).strip()


class TestSassFilter(AssetfilesTestCase):
    def test_processes_scss_files(self):
        self.mkfile(
            'static/css/simple.scss',
            '$c: red; body { color: $c; }')
        self.assertEquals(filter('css/simple.css'), 'body {\n  color: red; }')

    def test_processes_app_scss_files(self):
        self.mkfile(
            'app-1/static/css/app.scss',
            '$c: yellow; body { color: $c; }')
        self.assertEquals(filter('css/app.css'), 'body {\n  color: yellow; }')

    def test_processes_scss_files_with_deps(self):
        self.mkfile('static/css/folder/_dep.scss', '$c: black;')
        self.mkfile(
            'static/css/with_deps.scss',
            '@import "folder/dep"; body { color: $c; }')
        self.assertEquals(
            filter('css/with_deps.css'),
            'body {\n  color: black; }')

    def test_processes_scss_files_with_app_deps(self):
        self.mkfile('app-1/static/css/folder/_dep.scss', '$c: white;')
        self.mkfile(
            'static/css/with_app_deps.scss',
            '@import "folder/dep"; body { color: $c; }')
        self.assertEquals(
            filter('css/with_app_deps.css'),
            'body {\n  color: white; }')

    def test_integrates_static_url_with_sass(self):
        self.mkfile(
            'static/css/with_url.scss',
            'body { background: static-url("img/bg.jpg"); }')
        self.assertEquals(
            filter('css/with_url.css'),
            'body {\n  background: url("/static/img/bg.jpg"); }')

    def test_integrates_with_compass(self):
        self.mkfile(
            'static/css/with_compass.scss',
            '@import "compass"; .btn { @include border-radius(5px); }')
        self.assertIn('border-radius: 5px;', filter('css/with_compass.css'))

    def test_integrates_with_compass_image_url(self):
        self.mkfile(
            'static/css/image_url.scss',
            'body { background: image-url("img/bg.jpg"); }')
        self.mkfile(
            'static/css/only_path.scss',
            'body { background: url(image-url("img/bg.jpg", true)); }')
        self.mkfile(
            'static/css/cache_buster.scss',
            'body { background: image-url("img/bg.jpg", false, true); }')
        self.assertEquals(
            filter('css/image_url.css'),
            'body {\n  background: url("/static/img/bg.jpg"); }')
        self.assertEquals(
            filter('css/only_path.css'),
            'body {\n  background: url("/static/img/bg.jpg"); }')
        self.assertEquals(
            filter('css/cache_buster.css'),
            'body {\n  background: url("/static/img/bg.jpg"); }')

    def test_integrates_with_compass_font_url(self):
        self.mkfile(
            'static/css/font_url.scss',
            '@font-face { src: font-url("fonts/font.ttf"); }')
        self.mkfile(
            'static/css/only_path.scss',
            '@font-face { src: url(font-url("fonts/font.ttf", true)); }')
        self.assertEquals(
            filter('css/font_url.css'),
            '@font-face {\n  src: url("/static/fonts/font.ttf"); }')
        self.assertEquals(
            filter('css/only_path.css'),
            '@font-face {\n  src: url("/static/fonts/font.ttf"); }')

    def test_raises_syntax_error(self):
        with self.assertRaisesRegexp(
                SassFilterError,
                r'.*?Syntax error.*?\n.*?line 5.*?static/css/syntax_error\.scss'):
            self.mkfile('static/css/syntax_error.scss', '\n\n\n\nbody {')
            filter('css/syntax_error.css')


class TestCoffeeScriptFilter(AssetfilesTestCase):
    def test_processes_coffee_files(self):
        self.mkfile('static/js/simple.coffee', 'a = foo: "1#{2}3"')
        self.assertIn('foo: "1" + 2 + "3"', filter('js/simple.js'))

    def test_raises_syntax_error(self):
        with self.assertRaisesRegexp(
                CoffeeScriptFilterError,
                r'.*?SyntaxError.*?static/js/simple\.coffee.*?line 5'):
            self.mkfile('static/js/simple.coffee', '\n\n\n\na = foo: "1#{2}3')
            filter('js/simple.js')



import os

from django.conf import settings
from django.core import management
from django.utils import six

from assetfiles.tests.base import AssetfilesTestCase


def call_command(*args, **kwargs):
    out = six.StringIO()
    kwargs.update({'stdout': out, 'verbosity': 1})
    management.call_command(*args, **kwargs)
    out.seek(0)
    return out


class TestFindStatic(AssetfilesTestCase):
    def setUp(self):
        super(TestFindStatic, self).setUp()
        self.mkfile('static/css/main.css')
        self.mkfile('app-1/static/css/main.css')
        self.mkfile('app-2/static/css/main.css')

    def test_find_file(self):
        out = call_command('findstatic', 'css/main.css', all=False)
        lines = out.readlines()[1:]
        self.assertEquals(len(lines), 1)
        self.assertIn('project/static/css/main.css', lines[0])

    def test_find_all_files(self):
        out = call_command('findstatic', 'css/main.css')
        lines = out.readlines()[1:]
        self.assertEquals(len(lines), 3)
        self.assertIn('project/static/css/main.css', lines[0])
        self.assertIn('app-1/static/css/main.css', lines[1])
        self.assertIn('app-2/static/css/main.css', lines[2])


class TestCollectStatic(AssetfilesTestCase):
    def collectstatic(self):
        call_command('collectstatic', interactive=False, clear=True)

    def assertStaticFileNotFound(self, path):
        static_path = os.path.join(settings.STATIC_ROOT, path)
        self.assertFalse(os.path.isfile(static_path),
            "file '{0}' exists".format(path))

    def assertStaticFileExists(self, path):
        static_path = os.path.join(settings.STATIC_ROOT, path)
        self.assertTrue(os.path.isfile(static_path),
            "file '{0}' does not exist".format(path))

    def assertStaticFileContains(self, path, text):
        static_path = os.path.join(settings.STATIC_ROOT, path)

        self.assertStaticFileExists(path)
        with open(static_path, 'r') as file:
            self.assertIn(text, file.read(),
                "'{0}' not in file '{1}'".format(text, path))

    def test_copies_static_files(self):
        self.mkfile('static/css/static.css',
            'body { color: red; }')
        self.collectstatic()
        self.assertStaticFileContains('css/static.css',
            'body { color: red; }')

    def test_returns_app_static_files(self):
        self.mkfile('app-1/static/css/app_static.css',
            'body { color: blue; }')
        self.collectstatic()
        self.assertStaticFileContains('css/app_static.css',
            'body { color: blue; }')

    def test_processes_scss_files(self):
        self.mkfile('static/css/simple.scss',
            '$c: red; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileContains('css/simple.css',
            'body {\n  color: red; }')

    def test_processes_app_scss_files(self):
        self.mkfile('app-1/static/css/app.scss',
            '$c: yellow; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileContains('css/app.css',
            'body {\n  color: yellow; }')

    def test_processes_scss_files_with_deps(self):
        self.mkfile('static/css/folder/_dep.scss', '$c: black;')
        self.mkfile('static/css/with_deps.scss',
            '@import "folder/dep"; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileContains('css/with_deps.css',
            'body {\n  color: black; }')

    def test_processes_scss_files_with_app_deps(self):
        self.mkfile('app-1/static/css/folder/_dep.scss', '$c: white;')
        self.mkfile('static/css/with_app_deps.scss',
            '@import "folder/dep"; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileContains('css/with_app_deps.css',
            'body {\n  color: white; }')

    def test_skips_sass_dependencies(self):
        self.mkfile('static/css/_dep.scss', '$c: black;')
        self.mkfile('static/css/with_deps.scss',
            '@import "dep"; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileNotFound('css/_dep.css')
        self.assertStaticFileNotFound('css/_dep.scss')

    def test_processes_coffee_files(self):
        self.mkfile('static/js/simple.coffee', 'a = foo: "1#{2}3"')
        self.collectstatic()
        self.assertStaticFileContains('js/simple.js', 'foo: "1" + 2 + "3"')



import re


class BaseFilter(object):
    """
    Base class for filters that process files.
    """

    def matches_input(self, intput_path):
        """
        Returns true if the input file should be processed by this filter.

        Args:
            input_path: An absolute path to the file
        """
        return False

    def matches_output(self, output_path):
        """
        Returns true if the output file would be created by this filter.

        Args:
            output_path: An absolute path to the file
        """
        return False

    def possible_input_paths(self, output_path):
        """Returns possible input paths that would create the given output path.

        Args:
            output_path: An absolute path to the file
        Returns
            A set of search paths. Can be empty.

        """
        return set()

    def output_path(self, input_path):
        """
        Returns the path to the ouput file created with the given input path.

        Args:
            input_path: An absolute path to the file
        Returns:
            An absolute path to the output file.
        """
        return None

    def skip_output_path(self, output_path):
        """
        Determines wether to filter the file with the given output path.

        Implement this method if there are certain paths that should not
        be processed by this filter, i.e. they are dependencies.

        Args:
            output_path: An absolute path to the file.
        """
        return False

    def filter(self, input_path):
        """
        Filters the file with the given input path.

        All filters must implement this method.

        Args:
            input_path: An absolute path to the file to filter.

        Returns:
            The filtered string.
        """
        raise NotImplementedError()


class ExtFilter(object):
    """
    A mixin for filters that take a file with a certain file extension and
    output another.

    For example, SassFilter generally takes files with the exentions ".scss" and
    ".sass" and output CSS files (".css").

    Attributes:
        input_exts: A tuple of extensions (without the prefixed ".")
        output_ext: A single output extension (again, without the prefixed ".")
    """
    input_exts = None
    output_ext = None

    def __init__(self):
        if not self.input_exts and self.input_ext:
            self.input_exts = (self.input_ext,)

    def matches_input(self, intput_path):
        if self.input_exts:
            return re.search(r'\.({0})$'.format('|'.join(self.input_exts)), intput_path)
        return False

    def matches_output(self, output_path):
        if self.output_ext:
            return re.search(r'\.{0}$'.format(self.output_ext), output_path)
        return False

    def possible_input_paths(self, output_path):
        paths = set()
        if self.input_exts:
            for ext in self.input_exts:
                ext = '.' + ext
                paths.add(output_path + ext)
                paths.add(re.sub(r'\.[^\.]*$', ext, output_path))
        return paths

    def output_path(self, input_path):
        if self.output_ext:
            path = re.sub(r'\.{0}'.format(self.output_ext), '', input_path)
            ext = '.' + self.output_ext
            return re.sub(r'\.[^\.]*$', ext, path)
        return None

    def set_input_ext(self, value):
        self.input_exts = (value,)
    input_ext = property(fset=set_input_ext)



import os

from django.contrib.staticfiles.management.commands import collectstatic

from assetfiles import filters


class Command(collectstatic.Command):
    """
    Overrides staticfiles' `collectstatic` command to filter files before
    copying them to the target storage.
    """

    def copy_file(self, path, prefixed_path, source_storage):
        # Skip this file if it was already copied earlier
        if prefixed_path in self.copied_files:
            return self.log("Skipping '%s' (already copied earlier)" % path)
        # Delete the target file if needed or break
        if not self.delete_file(path, prefixed_path, source_storage):
            return
        # The full path of the source file
        source_path = source_storage.path(path)
        # Finally start copying
        if self.dry_run:
            self.log("Pretending to copy '%s'" % source_path, level=1)
        else:
            filter = filters.find_by_input_path(source_path)
            if filter:
                if filter.skip_output_path(prefixed_path): return

                self.log("Processing '%s'" % source_path, level=1)
                from assetfiles.storage import TempFilesStorage

                prefixed_path = filter.output_path(prefixed_path)
                content = filter.filter(source_path)
                source_storage = TempFilesStorage()
                source_storage.save(path, content)

            self.log("Copying '%s'" % source_path, level=1)
            if self.local:
                full_path = self.storage.path(prefixed_path)
                try:
                    os.makedirs(os.path.dirname(full_path))
                except OSError:
                    pass
            with source_storage.open(path) as source_file:
                self.storage.save(prefixed_path, source_file)
        if not prefixed_path in self.copied_files:
            self.copied_files.append(prefixed_path)



import os
import shutil
import tempfile

from django.conf import settings
from django.contrib.staticfiles import finders, storage
from django.test import TestCase
from django.utils.functional import empty

from assetfiles import filters
from assetfiles.filters.base import BaseFilter, ExtFilter
import assetfiles.settings


class AssetfilesTestCase(TestCase):
    def setUp(self):
        # Clear the cached staticfiles_storage out, this is because when it first
        # gets accessed (by some other test), it evaluates settings.STATIC_ROOT,
        # since we're planning on changing that we need to clear out the cache.
        storage.staticfiles_storage._wrapped = empty
        # Clear the cached staticfile finders, so they are reinitialized every
        # run and pick up changes in settings.STATICFILES_DIRS.
        finders._finders.clear()
        # Clear the cached assetfile filters, so they are reinitialized every
        # run and pick up changes in settings.ASSETFILES_FILTERS.
        filters._filters.clear()

        if not os.path.exists(settings.PROJECT_ROOT):
            shutil.copytree(
                os.path.join(settings.TESTS_ROOT, 'project-template'),
                settings.PROJECT_ROOT
            )
        self.addCleanup(shutil.rmtree, settings.PROJECT_ROOT, ignore_errors=True)
        self.root = settings.PROJECT_ROOT

    def mkfile(self, path, content=None):
        abspath = os.path.join(self.root, path)
        dirname = os.path.dirname(abspath)

        if not os.path.isdir(dirname): os.makedirs(dirname)

        with open(abspath, 'w') as file:
            if content: file.write(content)

        return abspath


class ReplaceFilter(ExtFilter, BaseFilter):
    input_exts = ('foo', 'baz')
    output_ext = 'bar'

    def __init__(self, pattern='a', replacement='b'):
        self.pattern = pattern
        self.replacement = replacement

    def filter(self, input):
        with open(input, 'r') as file:
            return file.read().replace(self.pattern, self.replacement)


class Filter1(ExtFilter, BaseFilter):
    input_exts = ('in', 'in1')
    output_ext = 'out'


class Filter2(ExtFilter, BaseFilter):
    input_ext = 'in2'
    output_ext = 'out2'
